@@ -27,20 +27,16 @@ import logging
 import os
 import posix
 import re
+import subprocess
 import sys
 import time
 import tempfile
 import threading
-# py3 compat
-try:
-    from urllib import url2pathname
-    url2pathname  # pyflakes
-except ImportError:
-    from urllib.request import url2pathname
+from urllib.request import url2pathname
 
 import gi
 gi.require_version("Gtk", "3.0")
-gi.require_version("Vte", "2.90")
+gi.require_version("Vte", "2.91")
 from gi.repository import Gtk
 from gi.repository import GObject
 from gi.repository import GLib
@@ -54,21 +50,12 @@ from gettext import gettext as _
 
 from .DebPackage import DebPackage
 from .SimpleGtkbuilderApp import SimpleGtkbuilderApp
-from .GDebiCommon import GDebiCommon, utf8
+from .GDebiCommon import GDebiCommon
 
 # the timeout when the termial is expanded if no activity from dpkg
 # is happening
 GDEBI_TERMINAL_TIMEOUT=4*60.0
 
-# HACK - there are two ubuntu specific patches, one for VTE, one
-#        for gksu
-UBUNTU=False
-try:
-    import lsb_release
-    UBUNTU = lsb_release.get_distro_information()['ID'] == 'Ubuntu'
-except Exception as e:
-    pass
-
 class GDebiGtk(SimpleGtkbuilderApp, GDebiCommon):
 
     def __init__(self, datadir, options, file=""):
@@ -80,7 +67,7 @@ class GDebiGtk(SimpleGtkbuilderApp, GDebiCommon):
         # use a nicer default icon
         icons = Gtk.IconTheme.get_default()
         try:
-          logo=icons.load_icon("gnome-mime-application-x-deb", 48, 0)
+          logo=icons.load_icon("package-x-generic", 48, 0)
           if logo != "":
             Gtk.Window.set_default_icon_list([logo])
         except Exception as e:
@@ -119,7 +106,8 @@ class GDebiGtk(SimpleGtkbuilderApp, GDebiCommon):
                                  self.on_window_main_drag_data_received)
 
         # Check file with gio
-        file = self.gio_copy_in_place(file)
+        if file != "":
+            file = self.gio_copy_in_place(file)
 
         #self.vte_terminal.set_font_from_string("monospace 10")
         self.cprogress = self.CacheProgressAdapter(self.progressbar_cache)
@@ -150,8 +138,6 @@ class GDebiGtk(SimpleGtkbuilderApp, GDebiCommon):
         if file != "" and os.path.exists(file):
             self.open(file)
 
-        self.window_main.set_sensitive(True)
-
     def _show_busy_cursor(self, show_busy_cursor):
         win = self.window_main.get_window()
         if not win:
@@ -236,7 +222,61 @@ class GDebiGtk(SimpleGtkbuilderApp, GDebiCommon):
             if path.endswith(".deb"):
                 self.open(path)
 
+    def _load_file_list_process(self, filenames):
+        err = 0
+        try:
+            for name in self._deb.control_filelist:
+                if name != "./":
+                    print(name, file=filenames, end='\0')
+            print(file=filenames, end='\0')
+            for name in self._deb.filelist:
+                if name != "./":
+                    print(name, file=filenames, end='\0')
+        except Exception as e:
+            logging.exception("Exception while reading the filelist: '%s'" % e)
+            err = 1
+        filenames.close()
+        return err
+
+    def _load_file_list_collect(self, condition, store, filenames):
+        if os.waitstatus_to_exitcode(condition) == 0:
+            store.clear()
+            header = store.append(None, [_("Package control data")])
+            filenames.seek(0)
+            for name in filenames.read()[:-1].split('\0'):
+                if name == "":
+                    header = store.append(None, [_("Upstream data")])
+                else:
+                    store.append(header, [name])
+            self.treeview_files.set_sensitive(True)
+        else:
+            store.clear()
+            store.append(None, [_("Error reading filelist")])
+        self.treeview_files.set_model(store)
+        self.treeview_files.expand_all()
+        filenames.close()
+
+    def _load_file_list(self):
+        filenames = os.fdopen(os.memfd_create('gdebi'), 'r+')
+        pid = os.fork()
+        if pid == 0:
+            os._exit(self._load_file_list_process(filenames))
+
+        store = Gtk.TreeStore(str)
+        self.treeview_files.set_sensitive(False)
+        store.append(None, [_("Reading filelist...")])
+        self.treeview_files.set_model(store)
+        GLib.child_watch_add(GLib.PRIORITY_DEFAULT, pid,
+                             lambda _, condition: self._load_file_list_collect(condition, store, filenames))
+
     def open(self, filename, downloaded=False):
+        self.window_main.set_sensitive(False)
+        self.statusbar_main.push(self.context,_("Loading..."))
+        self._open(filename, downloaded)
+        self.statusbar_main.push(self.context, "")
+        self.window_main.set_sensitive(True)
+
+    def _open(self, filename, downloaded):
         self._show_busy_cursor(True)
         res = GDebiCommon.open(self, filename, downloaded)
         self._show_busy_cursor(False)
@@ -245,8 +285,6 @@ class GDebiGtk(SimpleGtkbuilderApp, GDebiCommon):
                 Gtk.MessageType.ERROR, self.error_header, self.error_body)
             return False
 
-        self.statusbar_main.push(self.context, "")
-
         # set window title
         self.window_main.set_title(_("Package Installer - %s") %
                                    self._deb.pkgname)
@@ -260,7 +298,7 @@ class GDebiGtk(SimpleGtkbuilderApp, GDebiCommon):
         buf = self.textview_description.get_buffer()
         try:
             long_desc = ""
-            raw_desc = utf8(self._deb["Description"]).split("\n")
+            raw_desc = self._deb["Description"].split("\n")
             # append a newline to the summary in the first line
             summary = raw_desc[0]
             raw_desc[0] = ""
@@ -297,26 +335,13 @@ class GDebiGtk(SimpleGtkbuilderApp, GDebiCommon):
 
         # set various status bits
         self.label_version.set_text(self._deb["Version"])
-        self.label_maintainer.set_text(utf8(self._deb["Maintainer"]))
+        self.label_maintainer.set_text(self._deb["Maintainer"])
         self.label_priority.set_text(self._deb["Priority"])
-        self.label_section.set_text(utf8(self._deb["Section"]))
+        self.label_section.set_text(self._deb["Section"])
         self.label_size.set_text(self._deb["Installed-Size"] + " KiB")
 
         # set file list
-        store = Gtk.TreeStore(str)
-        try:
-            header = store.append(None, [_("Package control data")])
-            for name in self._deb.control_filelist:
-                store.append(header, [name])
-            header = store.append(None, [_("Upstream data")])
-            for name in self._deb.filelist:
-                store.append(header, [name])
-        except Exception as e:
-            logging.exception("Exception while reading the filelist: '%s'" % e)
-            store.clear()
-            store.append(None, [_("Error reading filelist")])
-        self.treeview_files.set_model(store)
-        self.treeview_files.expand_all()
+        self._load_file_list()
         # and the file content textview
         font_desc = Pango.FontDescription('monospace')
         self.textview_file_content.modify_font(font_desc)
@@ -418,8 +443,8 @@ class GDebiGtk(SimpleGtkbuilderApp, GDebiCommon):
             GLib.io_add_watch(channel, GLib.PRIORITY_DEFAULT,
                               GLib.IOCondition.IN | GLib.IO_ERR | GLib.IO_HUP,
                               self._on_lintian_output)
-            GLib.child_watch_add(GLib.PRIORITY_DEFAULT, pid,
-                                 self._on_lintian_finished)
+        GLib.child_watch_add(GLib.PRIORITY_DEFAULT, pid,
+                             self._on_lintian_finished)
 
     def _on_lintian_finished(self, pid, condition):
         exit_status = os.WEXITSTATUS(condition)
@@ -609,33 +634,16 @@ class GDebiGtk(SimpleGtkbuilderApp, GDebiCommon):
             if res != Gtk.ResponseType.YES:
                 return
 
-        if install:
-            msg_hdr = _("You need to grant administrative rights to install software")
-            msg_bdy = _("""
-It is a possible security risk to install packages files manually.
-Install software from trustworthy software distributors only.
-""")
-        else:
-            msg_hdr = _("You need to grant administrative rights to remove software")
-            msg_bdy = _("It is a possible risk to remove packages.")
         if os.getuid() != 0:
-
-            # build command and argument lists
-            gksu_cmd = "/usr/bin/gksu"
-            gksu_args = ["gksu", "--desktop",
-                         "/usr/share/applications/gdebi.desktop",
-                         "--message",
-                         "<big><b>%s</b></big>\n\n%s" % (msg_hdr,msg_bdy)]
-            gdebi_args = ["--", "gdebi-gtk", "--non-interactive",
+            pkexec_cmd = "/usr/bin/pkexec"
+            gdebi_args = ["gdebi-gtk", "--non-interactive",
                           self._deb.filename]
             if not install:
                 gdebi_args.append("--remove")
-            # check if we run on ubuntu and always ask for the password
-            # there - we would like to do that on debian too, but this
-            # gksu patch is only available on ubuntu currently unfortunately
-            if UBUNTU:
-                    gksu_args.append("--always-ask-pass")
-            os.execv(gksu_cmd, gksu_args+gdebi_args)
+            self.window_main.close()
+            while Gtk.events_pending():
+                Gtk.main_iteration()
+            sys.exit(subprocess.call([pkexec_cmd]+gdebi_args))
 
         if not self.try_acquire_lock():
             if install:
@@ -681,18 +689,18 @@ Install software from trustworthy software distributors only.
                                                     self.vte_terminal,
                                                     self.label_action,
                                                     self.expander_install)
-            #errMsg = None
             try:
                 res = self._cache.commit(fprogress,iprogress)
-            except IOError as msg:
+            except IOError as e:
                 res = False
-                #errMsg = "%s" % msg
+                msg = e
                 header = _("Could not download all required files")
                 body = _("Please check your internet connection or "
                         "installation medium, and make sure your "
                         "APT cache is up-to-date.")
-            except SystemError as msg:
+            except SystemError as e:
                 res = False
+                msg = e
                 header = _("Could not install all dependencies"),
                 body = _("Usually this is related to an error of the "
                         "software distributor. See the terminal window for "
@@ -866,21 +874,23 @@ Install software from trustworthy software distributors only.
             self.term_expander.set_expanded(False)
             self.install = install
         def commit(self):
-            def finish_dpkg(term, lock):
+            def finish_dpkg(term, status, lock):
                 """ helper that is run when dpkg finishes """
-                status = term.get_child_exit_status()
                 self.exitstatus = posix.WEXITSTATUS(status)
-                #print "dpkg finished %s %s" % (pid,status)
-                #print "exit status: %s" % self.exitstatus
-                #print "was signaled %s" % posix.WIFSIGNALED(status)
+                #print("dpkg finished %s %s" % (pid,status))
+                #print("exit status: %s" % self.exitstatus)
+                #print("was signaled %s" % posix.WIFSIGNALED(status))
                 try:
                     lock.release()
                 except:
                     logging.exception("lock.release failed")
 
             # get a lock
-            lock = threading.Lock()
-            lock.acquire()
+            try:
+                lock = threading.Lock()
+                lock.acquire()
+            except:
+                logging.exception("getting a lock failed")
 
             # ui
             if self.install:
@@ -905,10 +915,6 @@ Install software from trustworthy software distributors only.
 
             # the command
             argv = ["/usr/bin/dpkg", "--auto-deconfigure"]
-            # ubuntu supports VTE_PTY_KEEP_FD, see
-            # https://bugzilla.gnome.org/320128 for the upstream bug
-            if UBUNTU:
-                argv += ["--status-fd", "%s"%writefd]
             if self.install:
                 argv += ["-i", self.debfile]
             else:
@@ -922,7 +928,7 @@ Install software from trustworthy software distributors only.
 
             # prepare for the fork
             self.term.connect("child-exited", finish_dpkg, lock)
-            (res, pid) =self.term.fork_command_full(
+            (res, pid) =self.term.spawn_sync(
                 Vte.PtyFlags.DEFAULT,
                 "/",
                 argv,
@@ -931,6 +937,7 @@ Install software from trustworthy software distributors only.
                 # FIXME: add setup_func that closes all fds excpet for writefd
                 None, #setup_func
                 None, #setup_data
+                None, #cancellable
                 )
             #print "fork_command_full: ", res, pid
 
@@ -981,8 +988,7 @@ Install software from trustworthy software distributors only.
             self.env = ["VTE_PTY_KEEP_FD=%s"% self.writefd,
                         "DEBIAN_FRONTEND=gnome",
                         "APT_LISTCHANGES_FRONTEND=gtk"]
-        def child_exited(self,term):
-            status = term.get_child_exit_status()
+        def child_exited(self, term, status):
             #print "apt finished %s" % status
             #print "exit status: %s" % posix.WEXITSTATUS(status)
             #print "was signaled %s" % posix.WIFSIGNALED(status)
@@ -1015,7 +1021,7 @@ Install software from trustworthy software distributors only.
             # sleep just long enough to not create a busy loop
             time.sleep(0.01)
         def fork(self):
-            pty = Vte.Pty.new(Vte.PtyFlags.DEFAULT)
+            pty = Vte.Pty.new_sync(Vte.PtyFlags.DEFAULT)
             pid = os.fork()
             if pid == 0:
                 # *grumpf* workaround bug in vte here (gnome bug #588871)
@@ -1026,7 +1032,7 @@ Install software from trustworthy software distributors only.
                 pty.child_setup()
                 # FIXME: close all fds expect for self.writefd
             else:
-                self.term.set_pty_object(pty)
+                self.term.set_pty(pty)
                 self.term.watch_child(pid)
             return pid
         def wait_child(self):
